#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxStreamFile.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "cap.h"

int main (int argc, char** argv)
{
   alert_ELEM pdu;
   OSCTXT       ctxt;
   int          i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: reader [-v] [-i <filename>] [-notrace]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&ctxt, 1);

   /* Init PDU data type variable */

   Init_alert_ELEM (&ctxt, &pdu);

   /* Create input source object */

   stat = rtxStreamFileOpen (&ctxt, filename, OSRTSTRMF_INPUT);
   if (stat != 0) {
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Validate */

   if (0 == stat) {
      rtxStreamMark (&ctxt, INT_MAX);
      stat = XmlV_alert (&ctxt);
   }

   if (0 == stat) {
      if (trace) {
         printf ("Validation was successful\n");
      }
   }
   else {
      printf ("Validation failed\n");
      rtxErrPrint (&ctxt);
   }

   rtxErrReset (&ctxt);
   stat = 0;
   rtxStreamReset (&ctxt);

   /* Decode */

   if (0 == stat)
      stat = XmlD_alert (&ctxt, &pdu);

   if (0 == stat) {
      if (trace) {
         printf ("Decode was successful\n");
         printf ("Decoded record:\n");
         Print_alert_ELEM ("alert", &(pdu));
      }
   }
   else if (trace) {
      printf ("decode failed\n");
      rtxErrPrint (&ctxt);
   }

   rtxStreamClose (&ctxt);
   rtxFreeContext (&ctxt);

   return stat;
}
