/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "make-call.h"

int main (int argc, char** argv)
{
   MakeCall*    pMakeCall;
   OSCTXT	ctxt;
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   const OSOCTET* pMsgBuf;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose)
      rtxSetDiag (&ctxt, 1);

   /* Populate structure of generated type */

   pMakeCall = Test_MakeCall (&ctxt);

   /* Encode */

   stat = rtXmlSetEncBufPtr (&ctxt, 0, 0);

   if (0 == stat)
      stat = XmlE_MakeCall (&ctxt, pMakeCall);

   if (0 == stat) {
      pMsgBuf = rtXmlGetEncBufPtr (&ctxt);
      if (trace) {
         printf ("encoded XML message:\n");
         printf ("%s\n", (const char*)pMsgBuf);
      }
  
      /* Write the encoded message out to the output file */

  stat = rtXmlWriteToFile (&ctxt, filename);
  if (0 != stat) {
     printf ("Write to file %s failed\n", filename);
     rtxErrPrint (&ctxt);
  } 
   }
   else if (trace) {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
   }

   rtxFreeContext (&ctxt);
   return stat;

}
