/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/OSRTFileOutputStream.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "employee.h"

static void fill_jSmith (PersonnelRecord* pPR);

int main (int argc, char** argv)
{
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Create buffer and control objects

   PersonnelRecord value;
   OSXMLEncodeStream stream (new OSRTFileOutputStream (filename));

   if (stream.getStatus ()) {
      stream.printErrorInfo();
      return stream.getStatus ();
   }

   personnelRecord_CC pdu (stream, value);

   if (verbose) 
      rtxSetDiag (pdu.getCtxtPtr(), 1);

   // Populate structure of generated type

   fill_jSmith (&value);

   // Encode

   stat = pdu.encode();

   if (0 == stat) {
      if (trace) {
         printf ("XML message is encoded successfully.\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      stream.printErrorInfo();
      return stat;
   }

   return 0;
}

// Routine to fill the employee record with test data

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln);

static void fill_jSmith (PersonnelRecord* pPR)
{
   ChildInformation* pChildInfo;

   fill_Name (&pPR->name, "John", "P", "Smith");

   pPR->title.setValue ("Director");
   pPR->number.value = 51;
   pPR->dateOfHire.setValue ((const OSUTF8CHAR*)"19710917");

   fill_Name (&pPR->nameOfSpouse, "Mary", "T", "Smith");

   // fill first child record

   pChildInfo = new ChildInformation();
   fill_Name (&pChildInfo->name, "Ralph", "T", "Smith");
   pChildInfo->dateOfBirth.setValue ((const OSUTF8CHAR*)"19571111");
   pPR->children.append (pChildInfo);

   // fill second child record

   pChildInfo = new ChildInformation();
   fill_Name (&pChildInfo->name, "Susan", "B", "Jones");
   pChildInfo->dateOfBirth.setValue ((const OSUTF8CHAR*)"19590717");
   pPR->children.append (pChildInfo);
}

// Routine to fill a name structure

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln)
{
   pName->givenName.setValue (fn);
   pName->initial.setValue (mi);
   pName->familyName.setValue (ln);
}

