// This test driver program encodes a data record and writes the 
// encoded result to an output file..

#include <stdio.h>
#include <stdlib.h>
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "po.h"

int main (int argc, char** argv)
{
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   const OSOCTET* msgptr;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Create buffer and control objects

   PurchaseOrderType po;
   OSXMLEncodeBuffer buffer;
   purchaseOrder_CC pdu (buffer, po);

   buffer.setDiag (verbose);

   // Populate structure of generated type

   po.setOrderDate("2003-04-01");

   po.shipTo.setName("Joe Smith");
   po.shipTo.setStreet("1 Morning Light Rd");
   po.shipTo.setCity("Glenmoore");
   po.shipTo.setState("PA");
   po.shipTo.setZip(19343);

   po.billTo.setName("Bill Jones");
   po.billTo.setStreet("102 Pickering Way, Suite 506");
   po.billTo.setCity("Exton");
   po.billTo.setState("PA");
   po.billTo.setZip(19341);

   po.setComment("Need this ASAP!");

   Items_item* pItem = new Items_item();
   pItem->setProductName("Acme Widget");
   pItem->setQuantity(50);
   pItem->setUSPrice(200);
   pItem->setPartNum("111-AA");
   po.items.append (pItem);

   pItem = new Items_item();
   pItem->setProductName("Wizbang Gadget");
   pItem->setQuantity(99);
   pItem->setUSPrice(777);
   pItem->setComment("I would like these in the color red");
   pItem->setShipDate("2003-03-26");
   pItem->setPartNum("222-BB");
   po.items.append (pItem);

   // Encode

   stat = pdu.encode();

   if (0 == stat) {
      if (trace) {
         msgptr = buffer.getMsgPtr();
         printf ("encoded XML message:\n");
         printf ("%s\n", (const char*)msgptr);
      }
   }
   else {
      printf ("Encoding failed\n");
      buffer.printErrorInfo();
      return stat;
   }

   // Write the encoded message out to the output file

   if (buffer.write (filename) <= 0) {
      printf ("Can't write to %s\n", filename);
      return -1;
   }

   return 0;
}
