/* This test driver program reads an encoded record from a file         */
/* and decodes it and displays the results..                            */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxCharStr.h"
#include "rtxsrc/rtxStream.h"
#include "rtxsrc/OSRTSocketInputStream.h"
#include "employee.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"

#define DEFAULT_URL "http://127.0.0.1:9124"

int main (int argc, char** argv)
{
   int		i, numMsgs = 2, stat;
   OSBOOL       trace = TRUE, verbose = FALSE;
   int		port, destPort;
   const char*  url = DEFAULT_URL;
   char         buf[100], *address, *protocol;
   OSIPADDR     destIP;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-n")) numMsgs = atoi (argv[++i]);
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else if (!strcmp (argv[i], "-url")) {
            url = argv[++i];
         }
         else {
            printf ("usage: reader [ -v ] [ -i <filename>] [ -notrace ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -url  bind URL http://host:port\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   rtxStrcpy (buf, sizeof(buf), url);
   rtxSocketParseURL (buf, &protocol, &address, &port);

   OSRTSocket socket;

   stat = socket.bind (address, port);
   if (stat != 0) {
      printf ("bind failed, stat = %d\n", stat);
      return stat;
   }

   stat = socket.listen (1);
   if (stat != 0) {
      printf ("listen failed, stat = %d\n", stat);
      return stat;
   }

   if (trace) printf ("Wait for connection...\n");

   OSRTSocket* connectedSocket = socket.accept (&destIP, &destPort);

   if (connectedSocket == 0) {
      if (trace) printf ("Connection aborted. Status = %d\n", 
         socket.getStatus ());
      return socket.getStatus ();
   }

   if (trace) {
      char addr[100];

      if (OSRTSocket::addrToString (destIP, addr, sizeof (addr)) == 0)
         strcpy (addr, "<unknown>");

      printf ("Connection established with %s:%d\n", addr, destPort);
   }

   OSRTSocketInputStream in (*connectedSocket);
   OSXMLDecodeBuffer decodeBuffer (in);

   if (decodeBuffer.getStatus ()) {
      decodeBuffer.printErrorInfo();
      return decodeBuffer.getStatus ();
   }

   PersonnelRecord* pValue;

   // Decode

   printf ("decoding %d messages..\n", numMsgs);
   for (i = 0; i < numMsgs; i++) {
      pValue = new PersonnelRecord();
      personnelRecord_CC personnelRecord (decodeBuffer, *pValue);
      personnelRecord.setDiag (verbose);

      stat = personnelRecord.decode();

      if (0 == stat) {
         if (trace) {
            printf ("decoded XML message:\n");
            personnelRecord.print ("personnelRecord");
            printf ("\n");
         }
      }
      else {
         printf ("Decoding failed\n");
         decodeBuffer.printErrorInfo();
         delete pValue;
         return stat;
      }

      delete pValue;
   }

   return 0;
}

