import com.test.*;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.util.List;
import com.objsys.xbinder.runtime.*;

/**
 * This test program encodes a data record and writes the encoded result to an output file.
 */
public class Writer {

   private static DatatypeFactory dtFactory;

   static {
      try {
         dtFactory = DatatypeFactory.newInstance();
      }
      catch ( javax.xml.datatype.DatatypeConfigurationException dce ) 
      { throw new RuntimeException(dce); }
   }

   private static String outputFile = "message.xml";
   private static String charset = "UTF-8";     //default is UTF-8
   private static boolean trace = true;

   public static void main(String args[]) throws Exception 
   {
      //process command line to get the name of the output file
      //process command line to get the name of the input file
      for(int arg = 0; arg < args.length; arg++) {
         if ( args[arg].equals("-o")) {
           outputFile = args[++arg];
         }
         else if ( args[arg].equals("-notrace" ) ) trace = false;
         else {
            System.out.println("Usage: java Writer [-o <filename>] [-notrace]");
            System.out.println("  -o <filename> specifies the output file");
            System.out.println("  -notrace turns off echoing encoded data to stdout");
            System.exit(1);
         }
      }

      //create _CC object
      PurchaseOrder_CC root = new PurchaseOrder_CC();

      //Populate structures
      PurchaseOrderType purchaseOrder = new PurchaseOrderType();
      root.setPurchaseOrder(purchaseOrder);
      XMLGregorianCalendar orderDate = dtFactory.newXMLGregorianCalendarDate(2003, 04, 01, DatatypeConstants.FIELD_UNDEFINED);
      purchaseOrder.setOrderDate( orderDate );
      USAddress shipTo = new USAddress();
      shipTo.setName("Joe Smith");
      shipTo.setStreet("1 Morning Light Rd");
      shipTo.setCity("Glenmoore");
      shipTo.setState("PA");
      shipTo.setZip(19343);
      purchaseOrder.setShipTo(shipTo);
      USAddress billTo = new USAddress();
      billTo.setName("Bill Jones");
      billTo.setStreet("102 Pickering Way, Suite 506");
      billTo.setCity("Exton");
      billTo.setState("PA");
      billTo.setZip(19341);
      purchaseOrder.setBillTo(billTo);
      purchaseOrder.setComment("Need this ASAP!");

      Items items = new Items();
      purchaseOrder.setItems(items);
      List<Items_item> item = items.getItem();
      Items_item items_item = new Items_item();
      item.add(items_item);
      items_item.setPartNum("111-AA");
      items_item.setProductName("Acme Widget");
      items_item.setQuantity((byte)50);
      items_item.setUSPrice(200);

      items_item = new Items_item();
      item.add(items_item);
      items_item.setPartNum("222-BB");
      items_item.setProductName("Wizbang Gadget");
      items_item.setQuantity((byte)99);
      items_item.setUSPrice(777);
      items_item.setComment("I would like these in the color red");
      XMLGregorianCalendar shipDate = dtFactory.newXMLGregorianCalendarDate(2003, 03, 26, DatatypeConstants.FIELD_UNDEFINED);
      items_item.setShipDate(shipDate);


      //Encode data
      ByteArrayOutputStream bos = new ByteArrayOutputStream(32768);

      XBXmlEncoder encoder;
      encoder = new XBXmlEncoder(bos, charset);
      root.encodeDocument(encoder);
      encoder.close();

      if ( trace ) {
         //write encoded data to System.out
         System.out.println("Encoded data:\n");
         bos.writeTo(System.out);
         System.out.println();
      }

      //write encoded data to output file
      FileOutputStream fout = new FileOutputStream(outputFile);
      bos.writeTo(fout);
      fout.close();
   }
}
