/*
 * Copyright (c) 2017 Objective Systems, Inc.
 *
 * See the COPYING.txt file in this subdirectory for conditions on use
 * of this software.  This notice applies to software only in this
 * subdirectory (oortsrc) and not to other software that may be a
 * a part of this distribution.
 *
 *****************************************************************************/
/**
 * @file ooTrace.h
 * Diagnostic message trace functions.
 */
#ifndef _OOTRACE_H_
#define _OOTRACE_H_

#include <stdarg.h>
#include "oortsrc/ooCommon.h"

/* tracing */
#define OOTRCLVLERR     1
#define OOTRCLVLWARN    2
#define OOTRCLVLINFO    3
#define OOTRCLVLDBGA    4
#define OOTRCLVLDBGB    5
#define OOTRCLVLDBGC    6

#ifdef _OOWARNING
#define OOTRACELVL 2
#endif
#ifdef _OOINFO
#define OOTRACELVL 3
#endif
#ifdef _OODEBUGA
#define OOTRACELVL 4
#endif
#ifdef _OODEBUGB
#define OOTRACELVL 5
#endif
#ifdef _OODEBUGC
#define OOTRACELVL 6
#endif

/* Ensure we always log error messages */
#ifndef OOTRACELVL
#define OOTRACELVL 1
#endif

#define OOTRACEERR1(a)        ooTrace(OOTRCLVLERR,a)
#define OOTRACEERR2(a,b)      ooTrace(OOTRCLVLERR,a,b)
#define OOTRACEERR3(a,b,c)    ooTrace(OOTRCLVLERR,a,b,c)
#define OOTRACEERR4(a,b,c,d)  ooTrace(OOTRCLVLERR,a,b,c,d)
#define OOTRACEWARN1(a)       ooTrace(OOTRCLVLWARN,a)
#define OOTRACEWARN2(a,b)     ooTrace(OOTRCLVLWARN,a,b)
#define OOTRACEWARN3(a,b,c)   ooTrace(OOTRCLVLWARN,a,b,c)
#define OOTRACEWARN4(a,b,c,d) ooTrace(OOTRCLVLWARN,a,b,c,d)
#define OOTRACEINFO1(a)       ooTrace(OOTRCLVLINFO,a)
#define OOTRACEINFO2(a,b)     ooTrace(OOTRCLVLINFO,a,b)
#define OOTRACEINFO3(a,b,c)   ooTrace(OOTRCLVLINFO,a,b,c)
#define OOTRACEINFO4(a,b,c,d) ooTrace(OOTRCLVLINFO,a,b,c,d)
#define OOTRACEINFO5(a,b,c,d,e) ooTrace(OOTRCLVLINFO,a,b,c,d,e)
#define OOTRACEDBGA1(a)       ooTrace(OOTRCLVLDBGA,a)
#define OOTRACEDBGA2(a,b)     ooTrace(OOTRCLVLDBGA,a,b)
#define OOTRACEDBGA3(a,b,c)   ooTrace(OOTRCLVLDBGA,a,b,c)
#define OOTRACEDBGA4(a,b,c,d) ooTrace(OOTRCLVLDBGA,a,b,c,d)
#define OOTRACEDBGB1(a)       ooTrace(OOTRCLVLDBGB,a)
#define OOTRACEDBGB2(a,b)     ooTrace(OOTRCLVLDBGB,a,b)
#define OOTRACEDBGB3(a,b,c)   ooTrace(OOTRCLVLDBGB,a,b,c)
#define OOTRACEDBGB4(a,b,c,d) ooTrace(OOTRCLVLDBGB,a,b,c,d)
#define OOTRACEDBGC1(a)       ooTrace(OOTRCLVLDBGC,a)
#define OOTRACEDBGC2(a,b)     ooTrace(OOTRCLVLDBGC,a,b)
#define OOTRACEDBGC3(a,b,c)   ooTrace(OOTRCLVLDBGC,a,b,c)
#define OOTRACEDBGC4(a,b,c,d) ooTrace(OOTRCLVLDBGC,a,b,c,d)

#ifdef __cplusplus
extern "C" {
#endif

/**
 * This function opens a log file for output.
 *
 * @param filename     Name of file to be opened.
 * @param appendFlag   True = open file for append access.
 *                     False = open for write access (i.e. create new file)
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooTraceOpenLogFile (const char* filename, OOBOOL appendFlag);

/**
 * This function closes the currently open trace log file.
 */
EXTERN void ooTraceCloseLogFile ();

/**
 * This function sets the maximum trace level threshold.  The higher the
 * value, the more messages that will be displayed.
 *
 * @param traceLevel   Trace level threshold setting.  This should be set
 *                     to one of the OOTRCLVL* constants above.
 */
EXTERN void ooTraceSetThreshold (OOUINT32 traceLevel);

/**
 * This function formats and logs a trace message.  It will check the
 * given message trace level against the threshold to determine if the
 * message should be logged.
 *
 * @param traceLevel   Trace level.  This is compared to the current trace
 *                     level threshold to determine if the message should
 *                     be logged.
 * @param fmtspec      Format specification in printf format.
 */
EXTERN void ooTrace (OOUINT32 traceLevel, const char * fmtspec, ...);

/**
 * This function unconditionally writes the given messahe text to the
 * log file.  It is called by ooTrace.  Users should use ooTrace to
 * log messages to allow trace level filtering.
 * given message trace level against the threshold to determine if the
 * message should be logged.
 *
 * @param logMessage    Message to be logged.
 */
EXTERN void ooTraceLogMessage (const char* logMessage);

#ifdef __cplusplus
}
#endif
#endif
